/*********************************************************************
 *  SPDX-License-Identifier: MIT									 *
 *  Copyright (C) 2019-2024 by Helge Kortendieck  alan@kasmithee.de  *
 *                                                                   *
 *  This file is part of qublc.                                      *
 *********************************************************************/

#ifndef _QUBLC_H_
#define _QUBLC_H_

#include <QAction>
#include <QApplication>  // ???
#include <QDate>
//#include <QDesktopWidget>
#include <QCloseEvent>
#include <QFileDialog>
#include <QFileInfo>
#include <QHeaderView>
#include <QInputDialog>
#include <QList>
#include <QMainWindow>
#include <QMessageBox>
#include <QMenu>
#include <QMenuBar>
#include <QPoint>
#include <QRect>
#include <QScreen>
#include <QSettings>
#include <QSize>
#include <QStatusBar>
#include <QString>
#include <QStringList>
#include <QTabWidget>
#include <QTableWidgetItem>
#include <QTableView>
#include <QTimer>
#include <QToolBar>
#include <QWidget>
#include <QWindow>

#include "ui_qublc.h"			// main window
#include "abouthelp.h"			// about/help window
#include "rw.h"
#include "structs.h"
#include "ui_adddomain.h"		// add new domain entry dialog

#include <iostream>
using namespace std;

class Qublc : public QMainWindow		// MainWindow
{
Q_OBJECT

public:
	Qublc();
	    ~Qublc();

protected:
	void closeEvent(QCloseEvent *event);

private slots:

	void create_Actions();
	void create_Menus();
	void create_ToolBars();
	void create_StatusBar();
	void create_TableMenu();		// domain list widget right-click menu - add/delete entry

	void settings_Read();
	void settings_Write();

	void about();					// show about information
	void help();					// show help message
	void ahw_Create(int iD);		// create about/help/error dialog widget
	void ahw_Del();					// not showing the ahw widget at the moment

	void addDomainDlg_Create();		// create add new domain entry dialog
	void addDomainDlg_Clear();		// clear add new domain entry dialog
	void addDomainDlg_Hide();		// delete add new domain entry dialog
	void addDomainDlg_Show();		// show add new domain entry dialog
	void addDomainDlg_AddEntry();	// add new entry from add new domain dialog data

	void quit_Prep();				// quit clicked clean up
	void quit_Do();					// finally quit the program

	void domainListFile_New();								// create a new, empty domain list
	void domainListFile_OpenDlg();							// file dialog to get existing domain file
	bool domainListFile_Open();								// open domain list file
	void domainListFile_Save();								// save domain list
	void domainListFile_SaveAs();							// save domain list as
	bool domainListFile_Write(int iD, const QString& sD);	// save domain list to file and handle return info, iD=0=save, iD=1=save as

	bool domainListEntry_FindDomain(const QStringList& slD);	// check on input that a new domain (0=3rd, 1=2nd, 2=top level domain field) does not already exists - true == exists
	bool domainListEntry_ChkSeperator(QString& sD);		// check on input that the input text does not contain '|' , the data field seperator in the domain file
	void domainListEntry_Del();							// delete current domain entry
	bool domainListEntry_DelDo();						// delete a domain list entry
	void domainListEntry_Sort();						// sort domain list liDomains - 1st top level, then 2nd level then 3rd
	void domainListEntry_SortDlL1();					// sort domain list liDomains - either according top level (2) domain or 2nd level (1) domain entries
	void domainListEntry_SortDl2();						// sort domain list liDomains - 2nd level domain entries - get top level domain range(s)
	void domainListEntry_SortDl2Do();					// sort domain list liDomains - 2nd level domain entries per top level domain range(s)
	void domainListEntry_SortDl3();						// sort domain list liDomains - 3rd level domain entries - get top + 2nd level domain range(s)
	void domainListEntry_SortDl3Do();					// sort domain list liDomains - 3rd level domain entries per top + 2nd level domain range(s)
	void domainListEntry_SortDlByDate();				// sort domain list liDomains by date, oldest first

	int mbShow(int iD, const QString& sD, const QString& sD2, const QString& sD3, const QString& sD4, const QString& sD5);
	                                // show QMessageBox at cursor position, 1=Info, 2=Warning, 3=Critical, 4=Question, 5=Question yes/no/cancel
	                                // iD=type, sD=title, sD2=text, sD3=accept button text, sD4=reject button text, sD5=destructive button text (no but continue)

	void mbTxt_DomainListEntry_ChgReverted();				// show message box - this domain already exists, revert change - shown when table cells were changed
	int mbTxt_DomainListEntry_DelConfirm();					// show message box - confirm deletion of list entry
	void mbTxt_DomainListEntry_DelErr();					// show message box - error while trying to delete a domain list entry
	void mbTxt_DomainListEntry_DomainExists();				// show message box - domain already exists - shown when adding a new domain
	void mbTxt_DomainListEntry_InvalidInclude();			// show message box - wrong include input
	void mbTxt_DomainListEntry_FindDomainErr();				// show message box - find domain error - domain QStringList != 3 entries
	void mbTxt_DomainListEntry_SepRep();					// show message box - the data field separator '|' was entered and replaced by '/'

	void mbTxt_DomainListFileOpen_Empty();					// show message box - domain list is empty on open
	void mbTxt_DomainListFileOpen_FileOpenError();			// show message box - error - opening file
	void mbTxt_DomainListFileOpen_InvalidEntries(int iD2);	// show message box - error - ignored invalid entries
	void mbTxt_DomainListFileOpen_NoValidEntry();			// show message box - error - no valid entries
	void mbTxt_DomainListFileSave_Empty();					// show message box - domain list is empty on save or save as
	void mbTxt_DomainListFileSave_NotChanged();				// show message box - domain list was not changed on save or save as

	void mbTxt_Table_HeaderNothingImplemented();			// show message box - table header click not assigned to anything

	void mbTxt_UnboundFile_NoDomains();						// show message box - no domains - list is empty, no point to create file
	void mbTxt_UnboundFile_NoInclude();						// show message box - no entry to include in unbound file

	void table_CellChanged(int iD, int iD2);	// data of a cell has changed, check for '|', double domain entries and update underlying domain list
	void table_CellChangedConnect();			// connect signal: widget cell changed
	void table_CellChangedDisConnect();			// disconnect signal: widget cell changed
	void table_Clear();							// clear widget
	void table_MenuCalled(const QPoint& qPD);	// right click in table widget - call context menu to add/delete entry
	void table_Pop();							// populate widget
	void table_HeaderClicked(int iD);			// a table header was clicked - anything to do?
	void table_HeaderSet();						// set widget header
	void table_Upd();							// update widget (pop/clear)

	void timer5SaveHeadWidth();		// save header width of the qtablewidget

	void unboundFile_Create();		// save domain list to unbound readable file
	bool unboundFile_Write(int iD, const QString& sFile);	// write domain list to unbound readable file and handle return info, iD: 0=save 1=save as

	signals:
	    void sigAhwShow(int iD);	// show info dlg: 1=about, 2=help

	private:
	    Ui::Ublcw uiublcw;
		QWidget wUblcw;			// main window

		Ui::dW_AddDomain uiadddomain;
		QDialog wAddDomain;				// dialog add new domain entry

		bool bAhw;				// abouthelp widget created ?
		bool bBreak;			// to break or not to break
		bool bD;				// re-usable dummy bool
		bool bListChanged;		// list changed, ask to save on exit

		int iCurCol;			// column of current cell
		int iCurRow;			// row of current cell
		int iCurRowIx;			// index of row of current master cell (starting with 0 for 1st row)
		int iColWidth0;			// list widget: width column 0
		int iColWidth1;			// list widget: width column 1
		int iColWidth2;			// list widget: width column 2
		int iColWidth3;			// list widget: width column 3
		int iColWidth4;			// list widget: width column 4
		int iColWidth5;			// list widget: width column 5
		int iColWidth6;			// list widget: width column 6

		int i;
		int i2;
		int i3;
		int iBegin;				// begin of range
		int iBegin2;			// begin of range
		int iCnt;				// count (e. g. # list entries)
		int iCurCellCol;
		int iCurCellRow;
		int iD;					// dummy int
		int iD2;				// dummy int
		int iD3;				// dummy int
		int iD4;				// dummy int
		int iDomainListPos;		// position of domain definition in domain list - set after search
		int iEnd;				// end of range
		int iEnd2;				// end of range
		int iRet;				// return value of function (e. g. QMessageBox)
		int iSortDl;			// sort domain list according this column (1=2nd+3rd level, 2=top+2nd+3rd level domain)
		int iScrX;				// screen width
		int iScrY;				// screen heigth
		int iX;					// cursor position for messagebox
		int iY;					// cursor position for messagebox

		QAction *aboutAct;
		QAction *helpAct;
		QAction *exitAct;

		QAction *unboundFileCreateAct;

		QAction *domainListDomainAddAct;
		QAction *domainListDomainDelAct;
		QAction *domainListNewAct;
		QAction *domainListOpenAct;
		QAction *domainListSaveAct;
		QAction *domainListSaveAsAct;

		StrDomain strDomainD;			// struct entity of a single domain entry
		StrDomain strDomainD2;			// struct entity of a single domain entry
		StrDomain strDomainD3;			// struct entity of a single domain entry
		StrDomain strDomainMatch;		// struct entity of a single domain entry - found in search
		QList<StrDomain> liDomains;		// list mit domains to block

		QList<int> liIntD;				// integer list with selected rows from table selection

		QList<QTableWidgetItem *> liTwItemsD;	// list of selected table widget items

		QList<QTableWidgetSelectionRange> liSelRanD;  // list of selected ranges in table widget

		QMenu *exportMenu;				// export list as unbound *.conf file menu
		QMenu *fileMenu;				// file menu
		QMenu *helpMenu;				// help menu
		QMenu *menuDomainListRightClick;// domain list right-click menu - add/delete entry

		QPoint qPD;				// QPoint dummy
		QPoint qPos;			// cursor position

		QRect qrFrame;			// frame geometry = screen size

		QSize size;

		QString sChk;			// text to check
		QString sCur;			// current text
		QString sCur2;			// current text
		QString sD;				// dummy QString
		QString sD2;			// dummy QString
		QString sD3;			// dummy QString
		QString sD4;			// dummy QString
		QString sD5;			// dummy QString
		QString sDomainTop;		// top level domain entry
		QString sDomain2nd;		// 2nd level domain entry
		QString sDomain3rd;		// 3rd level domain entry
		QString sErrTit;		// error message titel
		QString sErrTxt;		// error message text
		QString sFile;				// file name (path+name)
		QString sFileDomain;		// current domain list file name (path+name)
		QString sFileDomainAs;		// new domain list file name
		QString sFileDomainLast;	// last saved domain list file name incl. path
		QString sFileUnbound;		// current unbound blacklist.conf file name (path+name)
		QString sFileUnboundAs;		// new unbound blacklist.conf file name
		QString sFileUnboundLast;	// last saved unbound blacklist.conf file name incl. path
		QString sNext;			// next text
		QString sPrev;			// previous text
		QString sPrev2;			// previous text

		QStringList slD;			// dummy QStringList
		QStringList slDomain;		// list with 3 domain entries, 0=3rd- , 1=2nd- , 2=top-level domain field
		QStringList slTH;			// widget table headers

		QTableWidgetItem *twItemD;	// table widget item

		QToolBar *fileToolBar;

		QTimer *timer5ListHead;		// 5 sec timer to store resizing of column width of the qtablewidgets

		aboutHelp *ahw;				// help widget
};

#endif 
